'use strict';

module.exports = function (grunt) {

  var _ = grunt.util._;

  require('load-grunt-tasks')(grunt);

  // Default task.
  grunt.registerTask('default', ['jshint', 'karma:unit']);
  grunt.registerTask('serve', ['karma:continuous', 'watch']);
  grunt.registerTask('build-doc', ['uglify', 'copy']);

  var testConfig = function (configFile, customOptions) {
    var options = { configFile: configFile, singleRun: true };
    var travisOptions = process.env.TRAVIS && { browsers: [ 'Firefox', 'PhantomJS'], reporters: ['dots'] };
    return grunt.util._.extend(options, customOptions, travisOptions);
  };

  // Demo dependencies
  var js_dependencies = [
    '<%= bower %>/angular-ui-bootstrap-bower/ui-bootstrap-tpls.min.js',
    '<%= bower %>/ace-builds/src-min-noconflict/ace.js'
  ];

  // Project configuration.
  grunt.initConfig({
    bower: 'bower_components',
    dist : '<%= bower %>/angular-ui-docs',
    pkg: grunt.file.readJSON('package.json'),
    meta: {
      banner: ['/**',
        ' * <%= pkg.name %> - <%= pkg.description %>',
        ' * @version v<%= pkg.version %> - <%= grunt.template.today("yyyy-mm-dd") %>',
        ' * @link <%= pkg.homepage %>',
        ' * @license <%= pkg.license %>',
        ' */',
        ''].join('\n'),
      view : {
        humaName : 'UI Ace',
        repoName : 'ui-ace',
        demoHTML : grunt.file.read('demo/demo.html'),
        demoJS : grunt.file.read('demo/demo.js'),
        css: ['assets/css/demo.css'],
        js: js_dependencies.concat(['build/ui-ace.min.js'])
      }
    },
    watch: {
      karma: {
        files: ['ui-ace.js', 'demo/**/*.js', 'gruntFile.js'],
        tasks: ['jshint', 'karma:unit:run'] //NOTE the :run flag
      }
    },
    karma: {
      unit: testConfig('test/karma.conf.js'),
      start: {configFile: 'test/karma.conf.js'},
      continuous: {configFile: 'test/karma.conf.js', background: true }
    },
    jshint:{
      files:['ui-ace.js', 'demo/**/*.js', 'gruntFile.js'],
      options: { jshintrc: '.jshintrc' }
    },
    uglify: {
      options: {banner: '<%= meta.banner %>'},
      build: {
        files: {
          '<%= dist %>/build/<%= meta.view.repoName %>.min.js': ['<%= meta.view.repoName %>.js']
        }
      }
    },
    copy: {
      main: {
        files: [
          {src: ['<%= meta.view.repoName %>.js'], dest: '<%= dist %>/build/<%= meta.view.repoName %>.js', filter: 'isFile'},
          {src: ['demo/demo.html'], dest: '<%= dist %>/demos.html', filter: 'isFile'},
          {src: ['<%= bower %>/ace-builds/src-min-noconflict/ace.js'], dest: '<%= dist %>/<%= bower %>/ace-builds/src-min-noconflict/ace.js', filter: 'isFile'}
        ]
      },
      template : {
        options : {processContent : function(content){
          return grunt.template.process(content);
        }},
        files: [
          {src: ['<%= dist %>/.tmpl/index.tmpl'], dest: '<%= dist %>/index.html'},
          {src: ['demo/demo.css'], dest: '<%= dist %>/assets/css/demo.css'}
        ]
          .concat(
            _.map(js_dependencies.concat([
              '<%= bower %>/ace-builds/src-min-noconflict/theme-twilight.js',
              '<%= bower %>/ace-builds/src-min-noconflict/mode-markdown.js',
              '<%= bower %>/ace-builds/src-min-noconflict/mode-scheme.js',
              '<%= bower %>/ace-builds/src-min-noconflict/worker-javascript.js'
            ]), function (f) {
              return {src: [f], dest: '<%= dist %>/' + f, filter: 'isFile'};
            }))
      }
    },
    changelog: {
      options: {
        dest: 'CHANGELOG.md'
      }
    }
  });

};
